module Takana
  class Middleware
    attr_reader :name, :path, :app, :kind, :paths, :host

    # TODO allow this to be toggled from configure block
    attr_accessor :debug

    alias_method :debug?, :debug

    def initialize(app, options={})
      @path = Dir.pwd.to_s
      @name = Pathname.new(@path).basename.to_s
      @app = app
      @kind, @paths = detect_kind!
      @debug = false
      @host = "localhost"
      @port = 48626

      if Takana.db

        if defined?(Rails)
          Rails.logger.debug "[Takana] #{@name} at #{@path} #{@kind} #{@paths}"
        end

        @project = Takana::Project[name: name] || Takana::Project.create(name: name, path: path, activated: false)

        @paths.each do |path|
          unless @project.load_paths.find { |_| _.path == path.to_s }.present? 
            @project.add_load_path(path: path.to_s, source: 'rack-takana')
          end

        end

        if defined?(Compass) && defined?(Rails) && Rails.application.config.compass.present?

          Rails.logger.debug "[Takana] detected compass. Attempting to load configuration"

          begin
            config = Rails.application.config.compass.serialize

            if config.present?
              attributes = {}

              Compass::Configuration::ATTRIBUTES.each do |k|
                attributes[k] = Rails.application.config.compass.send(k)
              end

              @project.compass_configs.each(&:delete)
              @project.add_compass_config(
                config: config,
                attributes: Marshal.dump(attributes)
              )
            end

            # tell projects to reload
            begin
              url = URI.parse("http://#{@host}:#{@port}/projects/load")
              req = Net::HTTP::Get.new(url.to_s)
              res = Net::HTTP.start(url.host, url.port) { |http| http.request(req) }
            rescue => e
              Rails.logger.debug "[Takana] project reload request failed, perhaps takana isn't running #{e.inspect}"
            end

          rescue => e
            Rails.logger.debug "[Takana] failed to load compass configuration"

          end
        end
      end
    end

    def call(env)
      @env = env

      @status, @headers, @response = @app.call(env)
      
      if is_takana_compatible_response?
        update_response!
        update_content_length!
      end

      [@status, @headers, @response]
    end

    private

    def detect_kind!
      if defined?(Rails) 
        paths = (Rails.application.config.sass[:load_paths] rescue []) + Rails.application.config.assets[:paths]

        ["rails", paths]
      else
        ["rack", []]
      end
    end

    def update_response!
      @response.each do |part|
        if is_regular_request? && is_html_response?
          insert_at = part.index('</body')
          unless insert_at.nil?
            part.insert(insert_at, render_takana_scripts)
          end
        end
      end
    end

    def update_content_length!
      new_size = 0
      @response.each { |part| new_size += part.bytesize }
      @headers.merge!("Content-Length" => new_size.to_s)
    end

    def is_regular_request?
      !is_ajax_request?
    end

    def is_ajax_request?
      @env.has_key?("HTTP_X_REQUESTED_WITH") && @env["HTTP_X_REQUESTED_WITH"] == "XMLHttpRequest"
    end

    def is_html_response?
      @headers["Content-Type"].include?("text/html") if @headers.has_key?("Content-Type")
    end

    def is_takana_compatible_response?
      return false if @status == 302
      return false if @env.has_key?("HTTP_SKIP_TAKANA_MIDDLEWARE")
      is_html_response?
    end

    def render_takana_scripts
      <<-EOT
        <script type="text/javascript" src="http://#{host}:48626/javascripts/edge.js" data-project="#{name}" data-debug="#{debug?}"></script>
      EOT
    end
  end
end